/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.AbstractForm");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

/**
 * Abstract object for handling basic editable form processing in RTLM. It provides
 * common edited fields handling and value retrieval from the form's fields.
 */
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.AbstractForm", null,
{   
    _editedList: null,

    _isLoading: false,
    
    // Summary
    // This method is called when fields in the form is edited by the user.
    // 
    _updateFormEditedState: function(w_id) 
    {
        // clear the status line if it's shown. This is the
        // case when we had just committed a change.
        this.clearStatusLine();
        
        // if the form fields are being loaded, we would get called as well.
        // Only act on the real editing by the user.    
        if(!this._isLoading) {
            var edited = this._isFieldChanged(w_id);
            this._setEditIndicator(w_id, edited);
            this._checkActionButtonEnablement();
        }
    },
   
    // Summary
    // Determines whether the field value has actually changed. The default implementation
    // acts like a "new" form. When a field has value, it is considered changed. Subclasses
    // needs to override this behavior to check the field values against a value previous cached.
    //   
    _isFieldChanged: function(w_id)
    {
        var node = dojo.byId(w_id);
        var newVal = this.getValueById(w_id);
        
        // Default implementation is to work for 
        // new form, as long as there's value
        // in the field, it's changed.
        if(newVal != null) {
            // if the object is a string, check for empty string.
            if(dojo.isString(newVal)) {
                return newVal.length > 0;
            }
            return true;
        }
        return false;
    },

    _checkHasRequireValue: function() 
    {
    },
    
    _checkActionButtonEnablement: function() 
    {
    },

    _checkValidInput: function() 
    {
        return true;
    },
        
    _checkClearStatusLine: function()
    {
        var node = dojo.byId(this.ACTION_STATUS_AREA_ID);
        if(node)
            return node.style.visibility == "visible";
        return false;
    }, 
    
    clearStatusLine: function() 
    {
        var node = dojo.byId(this.ACTION_STATUS_AREA_ID);
        node.innerHTML = "";
        dojo.style(node, "visibility", "hidden");
    },

    showStatusLine: function(message) 
    {
        var node = dojo.byId(this.ACTION_STATUS_AREA_ID);
        node.innerHTML = "<b>" + message + "</b>";
        dojo.style(node, "visibility", "visible");
    },
    
    isEdited: function() 
    {
        return(this._editedList != null && this._editedList.length > 0);
    },
    
    _isInEditedList: function(w_id)
    {
        if(this._editedList == null)
            return -1;
            
        for(var i = 0; i < this._editedList.length; i++) {
            if(this._editedList[i] == w_id)
                return i;
        }
        return -1;
    },
    
    // Summary
    // If certain fields shouldn't be decorated with the editing flag, this method should
    // return true. By default, all fields are not excluded. Subclasses need to override this
    // method to exclude certain fields to be decorated when edited.
    // 
    _excludeEditIndicator: function(w_id) 
    {
        return false;
    },
    
    // Summary
    // Change a field's appearance if the field has been edited. It also updates
    // the edited field list to reflect the current edited state of the form.
    //
    _setEditIndicator: function(w_id, edited) 
    {
        // only apply the edited indicator if we're told to
        // If this node is being excluded for edit indicator, 
        // don't do anything.
        if(!this._excludeEditIndicator(w_id)) {
            // modify the color scheme of the edited field as well.
            var node = dojo.byId(w_id);
            var baseName = this._getFieldDefaultClassName(w_id);
            var editedName = baseName+"Edited";
            
            // This call may be trigger from already edited to still edited
            // which the style for edited is already set.  If blindly replace
            // <baseName> to <baseName>"Edited" you might end up with
            // <baseName>"EditedEdited"
            if (edited) {
                // we are edited, check if we already have the editled class
                if (node.className.search(editedName) == -1) {
                    // if no then change from not edited to edited
                    node.className = node.className.replace(baseName, editedName);
                }
            } else {
                node.className = node.className.replace(editedName, baseName);
            }
        }
        this._updateEditList(w_id, edited);
    },
    
    _updateEditList: function(w_id, edited)
    {
        // update the edit list, we could add or remove the field
        // from the list.
        if(this._editedList == null)
            this._editedList = [];
        
        var inEditList = this._isInEditedList(w_id);
        if(edited) {
            if(inEditList < 0) {             
                this._editedList.push(w_id);
            }                
        }                
        else if(inEditList >= 0) {
            this._editedList.splice(inEditList, 1);
        }
    },
    
    // Summary
    // Resets all edited fields to be none edited. The edited list is also cleared.
    //
    _resetEditIndicators: function() 
    {
        if(this._editedList == null || this._editedList.length == 0)
            return;
        
        // the edit list would be modified as the fields are being 
        // set to un-edited. Start from the back to ensure the entire
        // list is traversed.    
        for(var i = this._editedList.length - 1; i >= 0 ; i--) {
            this._setEditIndicator(this._editedList[i], false);
        }
        this._editedList = null;
    },
    
    // Summary
    // Returns the field's class name. This is used to create the edited class name based on the
    // default class name.
    _getFieldDefaultClassName: function(w_id)
    {
    },
    
    // Summary
    // Retrieves the current value specified by the field id. For date fields (DateTextBox), it returns
    // the time in millisec in GMT.
    //  
    getValueById: function(id) 
    {
        var field = dijit.byId(id);
        
        // for date object, we always returns the date as a long,
        // i.e., the millisec since 1/1/1970 GMT
        if(field && field instanceof dijit.form.DateTextBox) {
			if(field.getValue() != null) {
                return field.getValue().getTime();
            }
        }
        else if(field && field instanceof dijit.form.TextBox) {
            if(field.domNode.value) {
                return field.getValue();
            }
        }
        else if(dojo.byId(id)) {
            var node = dojo.byId(id);
            
            if(node.type) {
	            if(node.type.indexOf("select") >= 0)
	                return this.getSelectValue(id);
	            else if(node.type.indexOf("checkbox") >= 0)
	                return node.checked;
	            else if(node.type.indexOf("textarea") >= 0)
	                return dojo.trim(node.value);
            }
        }
        return null;
    },
    
    
    getSelectValue: function(w_id) 
    {
        var combo = dojo.byId(w_id);
        if(combo && combo.options) {
            for(var j = 0; j < combo.options.length; j++) {
                if(combo.options[j].selected) {
                    return combo.options[j].text;
                }
            }
        }
        return "";
    },

    ACTION_STATUS_AREA_ID: "action_status"
});
