/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    Provides a progress monitor UI for any long running operations. Client should
 //    only interact with the ProgressUtility class and not the actual UI monitor
 //    panel (ProgressStatusMonitor). The ProgressUtility constructs the single monitor panel
 //    for any number of monitorable tasks.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ProgressBar");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.HoverPopup");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.DockingAnimator");

(function() {
    
var AsqUtil = com.ibm.asq.common.web.ui.internal.util.Utility;
var TrimStyles = jazz.ui.styledBox.TrimStyles;
var Package = com.ibm.asq.common.web.ui.internal.widgets.progressStatus;
var PreferenceStore = com.ibm.asq.common.web.client.internal.CommonPreferenceClient;
var DefaultProgressPanelPreference = PreferenceStore.USER_UI_CUSTOMISE_PROGRESS_PANEL_CATEGORY;


// The singleton monitor panel. We can only have one panel at a time.
var MonitorSingleton = null;

/*
 * Singleton interface to interact with the progress monitor panel. Client
 * should always use this 'static' class to interact with the progress monitor 
 * and should not create their own progress monitor panel.
 */
com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressUtility = {
    
    // Summary:
    //    Starts a task monitor display. If the task's size is known, a progress bar will
    //    be displayed and can be updated with a call to the updateTask method.
    //    Returns the task id for the task, any updates to the task needs to refer by the 
    //    returned id.
	//    Callback through the cancelCallback if the user selects cancel. 
    //    
    beginTaskMonitor: function(label, taskSize, cancelCallback, context)
    {
        if(!MonitorSingleton) {
            MonitorSingleton = new Package.ProgressStatusMonitor();
            MonitorSingleton.open(context);
        }
        var monId = MonitorSingleton.newMonitor(label, taskSize, cancelCallback);
        return monId;
    },

    // Summary:
    //    If the task size is known, call this method to update the task's progress by 
    //    specifying the amount of work done. If work done is equal to or greater then the
    //    taskSize, the endTaskMonitor method is auotmatically invoked.
    //
    updateTask: function(taskId, workDone)
    {
        if(!MonitorSingleton) {
            console.debug("Cannot update a task without first call begin task!");
            return;
        }
        MonitorSingleton.updateMonitor(taskId, workDone);   
    },
    
    // Summary:
    //   Call this method to end the progress monitor for the task. Optionally, specify an
    //   error message if the task was ended on an error condition.
    //
    endTaskMonitor: function(taskId, errorMessage)
    {
        if(!MonitorSingleton) {
            console.debug("Cannot update a task without first call begin task!");
            return;
        }
        MonitorSingleton.endMonitor(taskId, errorMessage);        
    }
};

/*
 * Simple action button widget for paging the progress monitors.
 */
var DisabledActionCss = "monitor-action-disabled";
dojo.declare("ProgressStatusMonitor.SimpleAction",
             [dijit._Widget, dijit._Templated],
{
    templateString: '<a href="javascript:{}" class="dijit dijitButtonNode monitor-action" waiRole="button" dojoAttachPoint="domNode" dojoAttachEvent="onclick:_handleClicked">' +
                    '  <img src="${_blankGif}" class="monitor-action-icon" dojoAttachPoint="iconLabelNode">' +
                    '  </img>' +
                    '</a>',
    label: null,
    iconClass: null,                    
    postCreate: function()
    {
        if(this.iconClass)
            dojo.addClass(this.iconLabelNode, this.iconClass);
        if(this.label)
            this.domNode.title = this.label;
    },
    setDisabled: function(disabled)
    {
        this.disabled = disabled;
        dojo.toggleClass(this.domNode, DisabledActionCss, this.disabled);        
    },
    onClick: function()
    {
    },
    _handleClicked: function(e)
    {        
        // don't let the event propagate up to chain. This may
        // trigger a 'link' clicked detection in the Jazz body
        // and would caused a dialog to popup saying 'are you sure
        // you want to leave' message. 
        dojo.stopEvent(e);

        if(this.disabled) {
            return;
        }
        this.onClick();
    }    
});

/*
 * The progress monitor widget. It can hold more then one monitor at a time. If more then
 * one monitor is shown, it creates a paging action to allow the user to scroll throught
 * the progress monitors. A monitor is created for each long running operation being added
 * to the monitor.
 */
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.ProgressStatusMonitor",
             [jazz.ui.StyledBox],
{    
    // our dialog's content template.
    contentTemplate:'<div dojoAttachPoint="contentNode" class="com-ibm-asq-common-ui-progress-monitor-content">' +
                    '  <div dojoAttachPoint="labelNode" class="label">' +
                    '  </div>' +
                    '  <div dojoAttachPoint="bodyNode" class="body">' +
                    '  </div>' +
                    '  <div dojoAttachPoint="footerNode" class="footer">' +
                    '    <span dojoAttachPoint="monitorSelectNode" class="monitor-select">' +
                    '       <span dojoAttachPoint="prevMonNode">' +
                    '       </span>' +
                    '       <span dojoAttachPoint="currMonText">' +
                    '       </span>' +
                    '       <span dojoAttachPoint="nextMonNode">' +
                    '       </span>' +
                    '    </span>' +
                    '    <span dojoAttachPoint="cancelNode" style="visibility:hidden">' +
                    '      <a href="javascript:{}" class="monitor-cancel" dojoAttachPoint="cancelLink">' +
                    '      </a>' +
                    '    </span>' +
                    '  </div>' +
                    '</div>',
  
    labelCancel: MessageUtil.getMessage(MessageBundle.COMMON, "actionLabel_cancel"),
    labelNext: MessageUtil.getMessage(MessageBundle.COMMON, "Next"),
    labelPrevious: MessageUtil.getMessage(MessageBundle.COMMON, "Previous"),
    secondaryTitle: "",
    headerless: true,
    trim: TrimStyles.BLUE_ALPHA_SHADOW,
    truncateTitle: false,
    closable: true,
    modal: false,
    
    postCreate: function() 
    {
        // attach the widgets created from the dom constructor to our object
        var dom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.contentTemplate);
        
        // supply the contentNode to the styled box, it
        // will inserted into its content area.
        this.contentNode = dom.contentNode;        
        this.inherited("postCreate", arguments);
                
        this.bodyNode = dom.bodyNode;
        this.footerNode = dom.footerNode;
        this.monitorSelectNode = dom.monitorSelectNode;
        this.currMonText = dom.currMonText;
        this.labelNode = dom.labelNode;
        this.cancelNode = dom.cancelNode;
        this.cancelLink = dom.cancelLink;
        
        this.cancelLink.innerHTML = this.labelCancel;
        this.connect(this.cancelLink, "onclick", "_cancelClicked");
        
        this.nMonitors = 0;
        this.monitors = {};
        
        this._addActions(dom);
        dojo.addClass(this.domNode, "com-ibm-asq-common-ui-progress-monitor");
        dojo.style(this.monitorSelectNode, "visibility", "hidden");
    },
    
    getAlignToPreference: function()
    {
        var pref = PreferenceStore.getStoredUserPreferences(DefaultProgressPanelPreference.key);
        if(!pref) {
            pref = DefaultProgressPanelPreference;
        }
        if(pref && pref.value && pref.value.alignTo) {
            return pref.value.alignTo;
        }
        return "alignLeft";
    },

    getAutoClosePreference: function()
    {
        var pref = PreferenceStore.getStoredUserPreferences(DefaultProgressPanelPreference.key);
        if(!pref) {
            pref = DefaultProgressPanelPreference;
        }
        if(pref && pref.value && pref.value.autoClose && !isNaN(pref.value.autoClose)) {
            return (parseInt(pref.value.autoClose) * 1000);
        }
        return -1;
    },
    
    open: function(parentWidget)
    {
        var d = dojo;
        var initTop = 200;
        var style = this.domNode.style;
        var vp = dijit.getViewport();
        var cs = d.getComputedStyle(this.domNode);
        var alignLeft = Package.DockingAnimator.alignLeft = (this.getAlignToPreference() == "alignLeft");
        
        // the computed style on IE returns 'auto'. Need to use a static value (from the CSS).
        var nodeInitWidth = isNaN(cs.width) ? 180 : parseInt(cs.width); 
        var leftPos = alignLeft ? 2 : (vp.w - nodeInitWidth - 1);
        
        if(parentWidget && parentWidget.domNode) {
            var coords = dojo.coords(parentWidget.domNode);
            initTop = coords.y + 20;
        }
        style.position = "absolute";
        style.display = "block";
        style.visibility = "visible";
        style.top = initTop + "px";
        style.left = leftPos + "px";
        
        d.style(this.domNode, "opacity", "0");
        d.body().appendChild(this.domNode);
        
        // fade in the monitor and then dock it
        d.fadeIn({
            node: this.domNode,
            duration: 600,
            onEnd: dojo.hitch(this, function(){
                Package.DockingAnimator.snapToDock(this);
            })
        }).play(0);
    },
    
    updateMonitor: function(id, workDone)
    {
        var monitor = this.monitors[id];
        
        if(!monitor) {
            console.debug("No such monitor: " + id);
            return;
        }
        // always put the monitor up in the stack when
        // it's being update
        monitor.update(workDone);
        this.showMonitor(monitor);
    },
    
    endMonitor: function(id, errorMessage)
    {
        var monitor = this.monitors[id];
        
        if(!monitor) {
            console.debug("No such monitor: " + id);
            return;
        }
        // always put the monitor up in the stack when
        // it's being update
        monitor.complete(errorMessage);        
        this.showMonitor(monitor);
    },
    
    newMonitor: function(label, jobSize, cancelCallback)
    {
        var monitor = new Package.TaskMonitor({
            label: label,
            taskSize: jobSize,
			cancelCallback: cancelCallback
        });
        
        this.nMonitors++;
        var id = this.nMonitors + "";
        this.monitors[id] = monitor;
        monitor.stackPos = this.nMonitors;
        this.showMonitor(monitor);
        return id;
    },
    
    showMonitor: function(monitor)
    {
        // we're already showing the same monitor
        // no need to bring in up in the stack.
        if (this.currentPos != monitor.stackPos) {
            dojox.data.dom.removeChildren(this.bodyNode);
            this.bodyNode.appendChild(monitor.domNode);
            this.labelNode.innerHTML = monitor.label;
            
            this.currentMonitor = monitor;
            this.currentPos = monitor.stackPos;
        }
        
        // turn on the cancel if the monitor is cancelable and not yet completed.
        dojo.style(this.cancelNode, "visibility", monitor.canCancel() && !monitor.isCompleted() ? "visible" : "hidden");
        
        if(this.nMonitors > 1) {
            dojo.style(this.monitorSelectNode, "visibility", "visible");
            this.currMonText.innerHTML = monitor.stackPos + " / " + this.nMonitors;
            
            this.prevAction.setDisabled(this.currentPos == 1);
            this.nextAction.setDisabled(this.currentPos == this.nMonitors);
        }
        
        // we have just update a monitor. If the preference is 
        // to auto close, check it now.
        var autoCloseIn = this.getAutoClosePreference();
        if(!this.closing && autoCloseIn > 0 && !this._hasActiveMonitor()) {
            // do auto close in x secs.
            this.closing = setTimeout(dojo.hitch(this, function() {
                delete this.closing;
                // close will consult w/ our onClose, if we have
                // a new monitor came in during close, we would
                // veto it.
                this.close();
            }), autoCloseIn);
        }
        else if(this.closing && this._hasActiveMonitor()) {
            // new status monitor came in and we are back in active.
            // don't auto close.
            clearTimeout(this.closing);
            delete this.closing;
        }
    },
    
    onClose: function() 
    {
        return !this._hasActiveMonitor();
    },
    
    _cancelClicked: function()
    {
        var monitor = this.currentMonitor;
        if(monitor) {
            monitor.invokeCancel();
            dojo.style(this.cancelNode, "visibility", "hidden");
        }
    },
    
    _addActions: function(dom)
    {
        var prevAction = new ProgressStatusMonitor.SimpleAction({
            label: this.labelPrevious,
            iconClass: "simple-prev-icon",
            onClick: dojo.hitch(this, function() {
                var pos = (this.currentPos - 1) + "";
                this.showMonitor(this.monitors[pos]);
            })
        });
        dom.prevMonNode.appendChild(prevAction.domNode);
        var nextAction = new ProgressStatusMonitor.SimpleAction({
            label: this.labelNext,
            iconClass: "simple-next-icon",
            onClick: dojo.hitch(this, function() {
                var pos = (this.currentPos + 1) + "";
                this.showMonitor(this.monitors[pos]);
            })            
        });
        dom.nextMonNode.appendChild(nextAction.domNode);
        
        this.prevAction = prevAction;
        this.nextAction = nextAction;
    },

    _hasActiveMonitor: function()
    {
        var monitors = this.monitors;
        
        for(var id in monitors) {
            if(!monitors[id].completed) {
                return true;
            }
        }
        return false;
    },
    
    // @override StyledBox - to better clean up our contents
    _onClose: function()
    {
        // the user can't wait and closing up. Clear the timer.
        if(this.closing) {
            clearTimeout(this.closing);
            delete this.closing;
        }
        
        var monitors = this.monitors;
        dojo.style(this.bodyNode, "visibility", "hidden");
        for(var id in monitors) {
            var w = monitors[id];
            if(w.stackPos != this.currentPos) {
                this.bodyNode.appendChild(w.domNode);
            }
        }
        AsqUtil.destroyWidgetsFromNode(this.domNode);
        this.inherited("_onClose", arguments);

        // clear our singleton
        MonitorSingleton = null;
    }
});

/*
 * Individual task monitor widget that lives inside main progress monitor panel. 
 * Each monitor has a progress bar that shows the progress of the corresponding
 * operation/task.
 */
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.progressStatus.TaskMonitor",
             [dijit._Widget, dijit._Templated],
{    
    templateString:'<div id="${id}" dojoAttachPoint="domNode" class="task-monitor">' +
                   '  <div dojoType="dijit.ProgressBar" style="width:97%" dojoAttachPoint="progressBar">' +
                   '  </div>' +
                   ' <div dojoAttachPoint="statusNode" style="display:none" class="progress-icon progress-image">' +
                   '   <span dojoAttachPoint="statusLabelNode" class="progress-label">${workingLabel}</span>' +					
                   '   <span dojoAttachPoint="detailsNode" style="display:none" class="error-detail-label">${detailsLabel}</span>' +                 
                   ' </div>' +
                   '</div>',
    
    workingLabel: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_working"),
    cancellingLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelCanceling"),
    canceledLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelCanceled"),
    completedLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelCompleted"),
    errorLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelOpFailed"),
    detailsLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelMoreDetails"),

    widgetsInTemplate: true,
    completed: false,
    taskSize: 0,
    label: "",
	cancelCallback: null,
	
    postCreate: function()
    {
        // this task's size is unknown, no way of showing any positive 
        // progress. Just show the working icon.
        if(!this.taskSize) {
            dojo.style(this.progressBar.domNode, "display", "none");
            dojo.style(this.statusNode, "display", "");
        }
        else {
            // initialize the progress bar.
            this.update(0);
        }
    },
    
    canCancel: function()
    {
        return (this.cancelCallback != null && dojo.isFunction(this.cancelCallback));
    },
    
    invokeCancel: function()
    {
        if(this.cancelCallback && dojo.isFunction(this.cancelCallback)) {
            this.cancelCallback();
            
            this.statusLabelNode.innerHTML = this.cancellingLabel;
            dojo.style(this.progressBar.domNode, "display", "none");
            dojo.style(this.statusNode, "display", "");
            
            this.inCancel = true;
        }    
    },
    
    update: function(worked)
    {
        if(!this.taskSize || this.completed) return;
        
        this.worked = worked;
        this.progressBar.update({
            maximum: this.taskSize,
            progress: this.worked
        });
        this.completed = (worked >= this.taskSize);
        if(this.completed) {
            this.complete();
        }
    },
    
    complete: function(errorMessage)
    {
        this.completed = true;
        dojo.style(this.progressBar.domNode, "display", "none");
        dojo.style(this.statusNode, "display", "");
        
        dojo.removeClass(this.statusNode, "progress-image");
        
        var icon = (errorMessage) ? "error-image" : "completed-image";
        dojo.addClass(this.statusNode, icon);
        this.statusLabelNode.className = "complete-label";
        
        var label = (this.inCancel) ? this.canceledLabel : ((errorMessage) ? this.errorLabel : this.completedLabel);
        this.statusLabelNode.innerHTML = label;
        
        if(errorMessage) {
        	dojo.addClass(this.statusLabelNode,"error-style");
            dojo.style(this.detailsNode, "display", "");
            this.tooltip = new jazz.ui.HoverPopup({html: errorMessage, around:[this.detailsNode]});
        }
    },
    
    isCompleted: function()
    {
        return this.completed;
    },

    uninitialize: function()
    {
        if(this.tooltip) {
            this.tooltip.destroy();
            delete this.tooltip;
        }
    }    
});

})();
