/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The floating status bar is shown when a table's footer is not visible
 //    due to the content's height exceeding the browser's viewport area. 
 //    It floats above the table's content and displays the statistics about
 //    the table's rows. 
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.FloatingStatusBar");

dojo.require("dojo.fx");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.util.ResizeNotifier");

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.FloatingStatusBar",
             [dijit._Widget, dijit._Templated],
{
    hostTableViewer: null,
    statusText: null,
    
    templateString: '<div class="floating-status-bar" dojoAttachPoint="domNode">' +
                    '  <span class="floating-status-bar-label" dojoAttachPoint="labelNode"></span>' +
                    '</div>',

    postCreate: function()
    {
        dojo.style(this.domNode, "display", "none");
        
        this.tableNode = this.hostTableViewer.domNode;
        this.tableFooterNode = this.hostTableViewer.footerAreaNode;

        this.tableNode.appendChild(this.domNode);

        // initialize our scroll window & floating pane parent, default
        // the scroll window is the main window.
        this.scrollWindow = window;
        this.rootFloatingPane = null;
        
        if(this.statusText) {
            this.setStatus(this.statusText);
        }
    },
    
    uninitialize: function()
    {
        this.tableNode = null;
        this.tableFooterNode = null;
        
        if(this.resizeHandler)
            jazz.util.ResizeNotifier.unregister(this.resizeHandler);
        if(this.scrollHandler)            
            dojo.disconnect(this.scrollHandler);
    },
    
    // Summary:
    //   Method to look for parent that is a floating pane (dialog). If our host table
    //   view lives inside a floating pane, we have to use the floating pane's content
    //   as the view port. 
    locateFloatPaneHost: function()
    {   
        // we only want/need to do this once.     
        if(this.lookedForHostFloatingPane) return;
        
        var node = this.tableNode;
        var widget = null;
        
        while (node) {
            var w = dijit.getEnclosingWidget(node);
            if (!w) break;
            
            // our parent is a floating pane.
			//Specific check for dijit.TooltipDialog for PopupFormField.TooltipDialog since its a temporary widget not abiding to jazz Dialogs
			//see FormToggleListField widget for description of PopupFormField.TooltipDialog 
            if (w instanceof jazz.ui.Dialog || w instanceof dijit.TooltipDialog) {
                widget = w;
                break;
            }
            // look further up.
            node = w.domNode ? w.domNode.parentNode : null;
        }
    
        this.lookedForHostFloatingPane = true;
        
        // we're living inside a floating pane. Record it
        // and use its container for triggering scrolling
        if(widget && widget.domNode) {
            this.rootFloatingPane = widget;
            if(widget.contentNode && widget.contentNode.parentNode) {
                this.scrollWindow = widget.contentNode.parentNode;
            }
        }
    },
    
    startup: function()
    {        
        this.scrollHandler = dojo.connect(this.scrollWindow, "onscroll", this, "_trackBottomPosition");
        this.resizeHandler = jazz.util.ResizeNotifier.register({
            context: this,
            // force browser resize changed notification on both horizontal & vertical
            force: true,
            callback: "_configure"
        });
        // update ourself when the floating pane has updated its position.
        // need to wait until it's done...
        if(this.rootFloatingPane && this.rootFloatingPane.updatePosition) {
            this.connect(this.rootFloatingPane, "updatePosition", function() {
                setTimeout(dojo.hitch(this, "_configure"), 100);
            });
        }
    },
    
    setStatus: function(st)
    {
        this.statusText = st;
        this.labelNode.innerHTML = st;
        
        if (!st || st.length == 0) {
            this._setActivate(false);
        }
        else if(!this.activated){
            this._configure();
        }
    },
    
    _setActivate: function(activate)
    {
        if(this.activated != activate)
            dojo.style(this.domNode, "display", activate ? "" : "none"); 
        this.activated = activate;
    },

    // Summary:
    //    initializes the floating bar's absolute X, Y position based
    //    on the host table's margin box. It automatically shows/hides
    //    the bar when needed.
    //
    _configure: function()
    {
        this.locateFloatPaneHost();
        
        var vp = dijit.getViewport();
        var mb = this.hostMb = dojo.coords(this.tableNode, true);
        var tableFooterH = this.tableFooterHeight = this.tableFooterNode ? dojo.marginBox(this.tableFooterNode).h : 40;
        var height = this.domNode.offsetHeight || 27;
                
        this.offsetToFooter = (mb.h + mb.y + mb.t - tableFooterH - height - 4);
        this.initX = mb.l - 3;
        this.initY = vp.h + mb.t - mb.y - height + 5;

        // we're living inside a floating pane. Our scroll window is not the
        // window anymore. We can just determine whether we need to activate
        // by just relative position of the host table and the hosting pane.
        if(this.rootFloatingPane) {
            mb = this.hostMb = dojo.marginBox(this.tableNode);
            var smb = dojo.marginBox(this.scrollWindow);
            
            // the viewport is the scroll window's box
            vp.h = smb.h;
            vp.t = this.scrollWindow.scrollTop;
            this.offsetToFooter = (mb.h - tableFooterH - height - 4);
            
            // But our positioning is still absolute. We need to add in the
            // top position of the pane. The dojo.coords() of the table doesn't
            // show that value.
            var slack = (dojo.isIE) ? 6 : 12;
            this.initY += (dojo.coords(this.rootFloatingPane.domNode, true).t - slack);
            this.initX += (dojo.isIE) ? 12 : 6;
        }
        
        var activate = (vp.h + vp.t < this.offsetToFooter);        
        this._setActivate(activate);
        
        // We are showing, configure the location/width
        // of the bar
        if (activate) {
            var style = this.domNode.style;
            style.top = this.initY + "px";
            style.left = this.initX + "px";
            style.width = mb.w + "px";
            
            this._trackBottomPosition();
            
            // we're activated, start tracking window scroll & resize
            if(!this.scrollHandler) {
                this.startup();
            }
        }
    },

   _trackBottomPosition: function()
    {
        // the widget would need to be configured first before
        // this method is called. This should never happens...
        if(!this.hostMb) return;
        
        var vp = dijit.getViewport();
        
        // We're inside the floating pane. Use the scroll window's 
        // margin box as the viewport.
        if(this.rootFloatingPane) {
            var mb = dojo.marginBox(this.scrollWindow);
            vp.h = mb.h;
            vp.t = this.scrollWindow.scrollTop;
        }
        var activate = (vp.h + vp.t < this.offsetToFooter);
        
        this._setActivate(activate);
        
        // not visible, just return now.
        if(!activate) {
            return;
        }
        
        // the viewport's t (top position) is the scroll position.
        // Anything positive we'll need to add to our original Y position
        // when the panel was first shown. 
        var newTop = this.initY + vp.t;
        this._anim = dojo.fx.slideTo({
            node: this.domNode,
            duration: 200,
            left: this.initX,
            top: newTop,
            onEnd: dojo.hitch(this, function() {
                delete this._anim;
            })
        }).play(0);        
    }
});

