/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The ListRowRenderer provides a reference implementation of a custom 
 //    renderer to draw a widget in each of the row in the table. The widget created by
 //    this row renderer is a simple more/less collapsible area. It is a "reference"
 //    implementation because it only draws the column data for the row inside the
 //    collapsible area. A useful row widget for a specific data model, needs to provide
 //    more data then the grid based row data and even actions in the opened area. 
 //    For that the client need to either provide a complete new implementation of the row 
 //    renderer, or at least subclass from this ListRowRenderer to add more contents 
 //    into the opened area.
 //
 //    One useful feature provided in this renderer is the caching of the collapsible
 //    widget's opened/closed state. When the TableViewer goes from one page to the next,
 //    the page's rows are deleted and recreated (through calling this object's renderRow() method). By
 //    caching the row object's widget open/close states, the display states of the rows are
 //    restored and maintained when the user is going from one page to the next, or when 
 //    performing sort and group actions. 
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility")
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableRowRenderer,
{
    cachedRowStates: null,
    
    constructor: function()
    {
        this.cachedRowStates = {};    
    },
    
    // Summary:
    //    Base interface method called by the TableViewer to create a row
    //    display data. This calls the class method _createListWidget() to
    //    construct a widget that will be placed inside the table row.
    //
    renderRow: function(viewer, rowObject, trNode)
    {
        var provider = viewer.getContentProvider();
        var td = document.createElement('td');
        var listWidget = this._createListWidget(viewer, rowObject);
        trNode.appendChild(td);
        
        td.colspan = provider.getNumColumns();
        td.width = "100%";
        td.appendChild(listWidget.domNode);
    },
    
    // Summary:
    //    Certain row renderer implementation might need to cache certain states about
    //    a row. The TableViewer will dynamically contruct and destruct rows as it's performing
    //    actions such as paging and sorting, etc. However, it's desirable for the row's states
    //    to be maintained across these actions. It's up to the row renderer to cache such
    //    states and re-apply them as the same row is being recreated. This method allows the 
    //    TableViewer to instruct the renderer to toss out any cached states, for events such as
    //    new contents is being set, etc.
    //
    clearRowStates: function()
    {
        this.cachedRowStates = {};        
    },
    
    // Summary:
    //    Creates a collapsible area widget that will be rendererd in the table's row. 
    //    To provide custom content to the list widget created in this method, subclasses
    //    should override the _createListContents() method.
    //
    _createListWidget: function(viewer, rowObject)
    {
        var provider = viewer.getContentProvider();
        var isOpen = this._getRowDisplayState(provider, rowObject);
		var self = this;
        var node = new com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea({
			open: isOpen, 
			//delay creation of the content to first time expand the list 
			OnFirstExpand: function(widget){
				self._createListContents(viewer, rowObject, node); //delay creating the content on first expand.
			} });
        
        this._createListTitle(viewer, rowObject, node);
        
        node.data = rowObject;
        node.provider = provider;
        node.onToggle = dojo.hitch(this, "_rowWidgetToggled");
        return node;
    },
	
	_createListTitle: function(viewer, rowObject, listWidget) {
		var provider = viewer.getContentProvider();
        var content = viewer.getContent();
        var colTitle = "";
        var openTitle = null;
        
        for(var i = 0; i < provider.getNumColumns(); i++) {
            var type = provider.getCellDisplayType(rowObject, i);
            
            if(type == TableConstants.RENDER_TYPE_TEXT ||
               type == TableConstants.RENDER_TYPE_DATE||
               type == TableConstants.RENDER_TYPE_LINK) {
                var data = provider.getCellDisplayData(rowObject, i);
				data = (!data) ? "" : data;
				var extData = provider.getCellDisplayExtendedInfo(rowObject, i);
                
                if(data instanceof Date) {
                    data = com.ibm.asq.common.web.ui.internal.util.Utility.formatDate(data);
                }
				
				if(type == TableConstants.RENDER_TYPE_LINK && extData && extData.href)
					data = "<a href='" + extData.href + "'>" + data + "</a>";
                
                // use the first column as the open title.
                if(openTitle == null) {
                    	openTitle = data;
                }
                else {
                    colTitle = colTitle + data;
                    
                    if(i < provider.getNumColumns() - 2) {
                        colTitle = colTitle + " | ";
                    }
                }
            }
        }
        
        if(openTitle) {
            listWidget.setTitle(openTitle);
        }
        listWidget.setCollapsedTitle(colTitle);
	},
    
    // Summary:
    //    Constructs simple contents based on the content provider's column data. Subclasses
    //    that want to use the same list widget created by this class, should override this
    //    method to supply custom contents in the list for the row object. 
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        var provider = viewer.getContentProvider();
        var content = viewer.getContent();
        var text = "";
        
		var table = document.createElement("table");
        for(var i = 0; i < provider.getNumColumns(); i++) {
            var type = provider.getCellDisplayType(rowObject, i);
            
            if(type == TableConstants.RENDER_TYPE_TEXT ||
               type == TableConstants.RENDER_TYPE_DATE||
               type == TableConstants.RENDER_TYPE_LINK) {
                var title = provider.getColumnHeading(content, i);
                var data = provider.getCellDisplayData(rowObject, i);
				data = (!data) ? "" : data;
				var extData = provider.getCellDisplayExtendedInfo(rowObject, i);
                
                if(data instanceof Date) {
                    data = com.ibm.asq.common.web.ui.internal.util.Utility.formatDate(data);
                }
				
				if(type == TableConstants.RENDER_TYPE_LINK && extData && extData.href)
					data = "<a href='" + extData.href + "'>" + data + "</a>";
				
				var tr = table.insertRow(table.rows.length);
				tr.insertCell(0).innerHTML = "<b>" + title + ":</b>";
				tr.insertCell(1).innerHTML = data;
            }
        }

        listWidget.contentNode.appendChild(table);
    },
    
    _getRowDisplayState: function(provider, rowData)
    {
        var isOpen = false;
        var key = provider.getRowDataId(rowData);
        
        if(key) {
            if(typeof this.cachedRowStates[key] != "undefined") {
                isOpen = this.cachedRowStates[key];
            }
        }
        return isOpen;
    },

    _setRowDisplayState: function(provider, rowData, isOpen)
    {
        var key = provider.getRowDataId(rowData);
        
        if(key) {
            this.cachedRowStates[key] = isOpen;
        }        
    },

    // Summary:
    //    Records the list widget's open/close state. 
    // 
    _rowWidgetToggled: function(widget)
    {
        this._setRowDisplayState(widget.provider, widget.data, widget.open);
    }
});

})();
