/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The Table Content Provider is the interface class that the client of the TableViewer
 //    must implement and set to the TableViewer. The table viewer uses the provider to obtain
 //    the displayable information about the contents. The client's content provider implementation
 //    must be a subclass of this base interface class and must implement all of the methods 
 //    defined here. However, if a specific client doesn't concert about certain display control
 //    information, such as the cell alignment, the client's subclass doesn't need to override
 //    those methods. Each method defined below will indicate whether the client should/must 
 //    override in their implementation.
 //    
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants; 

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider", null,
{
    // Summary:
    //    Returns the number of columns to be displayed in the TableViewer for the set content
    //    in the viewer.
    //    
    //    Client must implement this method to provide the number of columns to be displayed in the table.
    //
    getNumColumns: function(content)
    {
        return 0;
    },
	
    // Summary:
    //    Returns the column heading for the specified column index. The returned value must be
    //    non-null.
    //    
    //    Client must implement this method to provide the specific heading for each column.
    //
    getColumnHeading: function(content, atColumnIndex)
    {
        return "";
    },

    // Summary:
    //    Returns the id name of the column. The ID name is the server side identifier
    //    of the column. If this name matches the ID name of a filter in the filter provider,
    //    the column will have the filter display for it.
    //
    getColumnIdName: function(atColumnIndex)
    {
        return null;
    },
    
    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    //    Client optionally can supply an image for a column as well as making it a click-able action.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
        /*
         * returns the image src string. for example: 
         *     return dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/templates/common/images/image.gif");
         *   
         * Or return an object to indicate it's an action for example:
         *     return {actionId: "actionName", tooltip: "click me!", src: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/templates/common/images/action.gif")};
         */
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return TableConstants.ALIGN_LEFT;
    },	 
    
    // Summary:
    //    Returns whether the column can be used as a grouping control. 
    //    
    //    Client optionally can indicate whether a column can be grouped.
    //
    getIsColumnGroupable: function(content, atColumnIndex)
    {   
        return false;
    },

    // Summary:
    //    Returns whether the column can be used as a sorting control.
    //    
    //    Client optionally can indicate whether a column can be sorted.
    //
    getIsColumnSortable: function(content, atColumnIndex)
    {
        return false;
    },
    
    // Summary:
    //    Returns the column's width in integer percentage. All columns in the table should
    //    be added up to 100.
    //    
    //    Client optionally can indicate the width of a column.
    // 
    getColumnWidth: function(content, atColumnIndex)
    {
        // for example, return 20;
    },
	    
	// Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //    
    //    Client must implement this method to provide row objects for the table.
    //
    getContentObjects: function(content)
    {
        // returns the rows data of the content (the data), can not be non-null
        return [];
    },
    
    // Summary:
    //    Returns the cell data's horizontal alignment. By default, the cell data are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the cell.
    //
    getCellAlignment: function(rowData, atColumnIndex)
    {
        return TableConstants.ALIGN_LEFT;
    },

    // Summary:
    //    Returns the unique id string for the row. If this data is supplied, the
    //    display states of the row can be cached. For example, if the table is set
    //    to "List" mode, and the individual rows can be collapsed, or expanded. 
    //    If an unique Id of the row is provided, the collapsed/expanded state of the
    //    rows can be kept. In the situations where the rows must be recreated in
    //    sorting/paging, etc., the state of the row can be preserved, such as the 
    //    checked state.
    //    
    //    Client *should* implement this method to provide an unique id for each row object.
    //
    getRowDataId: function(rowData)
    {
        return null;
    },

    // Summary:
    //    Returns a displayable name for the row data.
    //
    //    Client optionally can provide a displayable name for the row data.
    //
    getRowDataName: function(rowData)
    {
        return null;
    },

    // Summary:
    //    Returns whether the cell is editable.  This will be ignored if
    //    the table viewer is set to be non-editable.
    //    
    getIsCellEditable: function(rowData, atColumnIndex)
    {
        return true;
    },
    
	// Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer to draw the cell information.
    //    
    //    Client must implement this method to provide a registered cell renderer 
    //    type for the column display for the row object.
    //
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        return TableConstants.RENDER_TYPE_TEXT;
    },
    
	// Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {   
       return "";
    },

    // Summary:
    //    Returns the raw data of the cell. This method is only used by the sort/group/filter operations
    //    of the table to speed up performance. If there're additional processing that needs to be done
    //    on the display data, like word breaks/wrap in text, it would be faster if these operations can
    //    by-pass that. The sort/group/filter operations would first look at the data provided by this method
    //    first. If no data is provided, those operations would revert to using the getCellDisplayData. 
    //    The clients are encouraged to implement this method if the display data needs extra massaging. However,
    //    the returned raw data shouldn't affect the outcome of the sort/filter operations. In other words, the
    //    'raw' data needs to be identical to the display data. Date cell type would be benefit to return just
    //    the long value of the time instead of converting the long to a Date object for the display.
    //
    //    Client should implement this method if the cell data requires additional processing for preparing the display data
    //    and that the content of the cell data would be identical in raw form.  
    //    
    getCellRawData: function(rowData, atColumnIndex)
    {   
       return undefined;
    },
    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the default selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'},... ] array of objects with 'text' and 'value' attributes
    //
    //        NOTE: if the client doesn't provide the 'select' attribute in the extended data, the TableViewer (the default 
    //        ComboBoxCellRenderer) will use the value returned by getCellDisplayData().
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //
    //      /* attributes for DROPDOWN_MENU cell renderer type */
    //      "actions" : [{'id':'the id of the action to be passed to the cell modifier', 'label':'action name label', 'iconClass': 'iconclassName'},... ] array of action objects.
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {   
       return null;
    },
	
	getGroupColumnExtraData: function(groupContent, atColumnIndex)
	{
		return null;
	},
    
    // Summary:
    //    This method is called whenever a new content is set to the table viewer. 
    //
    contentChangedNotify: function(content)
    {
        // receives content changed event.
    },
    
    // Summary:
    //    Returns whether the content is virtual, meaning that only partial content is
    //    loaded locally on the client. If this method returns true, the table viewer will delegate
    //    all content related processing to the provider using the following methods prefixed by 'vc'.
    //
    //    Note: Only clients with virtual content need to implement this method and the 'vc' methods.
    //    
    isVirtualContent: function()
    {
        // returns true if the content is virtual.
        return false;
    },
    
    // Summary:
    //    Returns the total virtual content count. If the content's count is unknown, the client
    //    should return the maximum page size. This allows the table to create all of the selectable
    //    page sizes for the user. 
    //
    vcGetContentItemCount: function()
    {    
        return TableConstants.MAX_PAGE_SIZE;
    },
    
    // Summary:
    //    Set and change the visible page size of the content. 
    //
    vcSetPageSize: function(newPageSize)
    {    
    },
     
    // Summary:
    //    Returns the total virtual content's page count.
    //
    vcGetTotalPageCount: function()
    {    
    },
    
    // Summary:
    //    Returns the current page's index value 0 based, i.e., 0 to total page count - 1
    //
    vcGetCurrentPageIndex: function()
    {    
    },
    
    // Summary:
    //    Returns true if the virtual content has next page. 
    //
    vcGetHasNextPage: function()
    {
    },

    // Summary:
    //    Returns true if the virtual content has previous page. 
    //
    vcGetHasPreviousPage: function()
    {
    },
         
    // Summary:
    //    Display the first page of the virtual content
    //
    vcPerformFirstPage:function()
    {
    },
    
    // Summary:
    //    Display the last page of the virtual content
    //
    vcPerformLastPage: function()
    {
    },

    // Summary:
    //    Display the previous page of the virtual content
    //
    vcPerformPreviousPage: function()
    {
    },

    // Summary:
    //    Display the next page of the virtual content
    //
    vcPerformNextPage: function()
    {
    },
    
    // Summary:
    //    Display the specified page of the virtual content
    //
    vcPerformIndexPage: function(pageIndex)
    {
    },
    
    // Summary:
    //    Single sort the virtual content on the specified column index
    //
    vcPerformSortBy: function(colIndex, sortDir)
    {
    },
    
    // Summary:
    //    Group the virtual content on the specified column index
    //
    vcPerformGroupBy: function(colIndex)
    {
    },

    // Summary:
    //    Muiti-column sort the virtual content on the specified columns.
    //    SortColumns is expected to be array holding the sort columns in order, 0 = 1st, 1 = 2nd, etc.
    //
    //     [ 
    //         {'col': column position, 'dir': direction},
    //         ...
    //     ]
    //
    vcPerformMultiColumnSort: function(sortColumns)
    {
    },
    
    // Summary:
    //    Filter virtual content by free form text filter.
    //
    vcPerformFilterBy: function(filterText)
    {
    },
    
    // Summary:
    //    Compose the server query arguments for table's sort and group settings.
    //
    buildGroupSortServerArgs: function()
    {
    },
    
    // Summary:
    //    Returns the current free form text filter.
    //
    getTextFilter: function()
    {
        return this.filterText;    
    }        
});

})();
