/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The TableHeaderRenderer is an internal class to the TableViewer. It's responsible
 //    for constructing the user controls for the table. This is an abstract base class for 
 //    two concrete header renderer classes: one for the GRID mode, which has a table heading
 //    and the other for the LIST mode. This abstract base classes provides methods 
 //    to create common controls for both table modes. Clients of the TableViewer don't have the need
 //    to customize the TableViewer's header and controls and thus, it doesn't have 
 //    APIs to set/get a TableHeaderRenderer at this point.
 // 
 //    This base class provides methods to construct the table's paging and filtering controls.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

dojo.require("com.ibm.asq.common.web.client.internal.Constants");

( function() {
var _textContent=dojo.isIE?"innerText":"textContent"; 

var Constants = com.ibm.asq.common.web.client.internal.Constants;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var FilterInputDelay = 800;
var MaxNumPageIndicesDisplay = 7;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableHeaderRenderer", null,
{
    // the parent viewer that employs this header renderer
    viewer: null,
    sortByColumn: -1,
    sortDir: TableConstants.SORT_DIR_ASCEND,
    
    // group controls
    groupSelect: null,
    groupByColumn: -1,
    
    // page controls
    pageCtrlPrev: null,
    pageCtrlInfo: null,
    pageCtrlNext: null,
    pageCtrlFirst:null,
    pageCtrlLast:null, 
    
    
    // filter controls
    filterInput: null,
    resetFilter: null,
	    
    isCJK: false,
	tableViewer_showAllItems: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_showAllItems"),
	tableViewer_itemsPerPage: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_itemsPerPage"),
	tableViewer_groupBy: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_groupBy"),
	tableViewer_ungrouped: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_ungrouped"),
	tableViewer_viewModeSelectLabel: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_viewModeSelectLabel"),
	labelPageRange: Constants.getMessage("labelPageRange"),
	labelNext: Constants.getMessage("Next"), 
	labelPrevious: Constants.getMessage("Previous"), 
	labelFilterInput: Constants.getMessage("tableViewer_filterInputLabel"),
    labelFirstPage: Constants.getMessage("tableViewer_firstPage"),
    labelLastPage: Constants.getMessage("tableViewer_lastPage"),
    labelGotoPage: Constants.getMessage("tableViewer_gotoPage"),
    labelFilterPrompt: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_filterPrompt"),
    labelFilterPromptPageMode: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_filterPromptPageMode"),
    labelFilterPromptDesc: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_filterPromptPageModeDesc"),
    
    
    constructor: function(viewer)
    {
        this.viewer = viewer;
        this.pageCtrlPrev = [];
        this.pageCtrlInfo = [];
        this.pageCtrlNext = [];
        this.pageCtrlFirst= [];
        this.pageCtrlLast = [];
        
        this.delayFilterTimer = null;
    
        // if this is CJK locale add in our cjk locale specific CSS
        this.isCJK = com.ibm.asq.common.web.ui.internal.util.Utility.isCJKLocale();
    },
    
    // Summary:
    //    Reset any states that the header may have cached. This is usually called when
    //    the table viewer's content has been changed and any old state may not be applicable.
    //
    resetStates: function()
    {
        // base class have no cached states, do nothing.
    },
    
    // Summary:
    //   Creates the TableViewer's controls. The TableViewer calls this renderer method
    //   to create the controls for its view.
    //
    createTableControls: function(pageRangeSelectNode, pageNode, footerPageNode, filterControlNode, groupControlNode, viewModeControlNode)
    {
        
    },

    destroyTableControls: function()
    {
        this.disconnectAll();
    },
    
    // Summary:
    //    The table header might needs to be reconstructed many times during the table's
    //    active life cycle. Before calling the createTableHeader method, call the destroy
    //    header first to clean up the old table header. Subclasses needs to implement this
    //    method and call this.inherited(arguments);
    destroyTableHeader: function()
    {
    },
         
    // Summary:
    //   Creates the TableViewer's header. The TableViewer calls this renderer method
    //   to create the header for the table.
    //
    createTableHeader: function(table)
    {
    },
    
    _handlePageIndexClicked: function(e) 
    {
        // local event don't propagate the event up.
        dojo.stopEvent(e);
       
        var name = (e.currentTarget) ? e.currentTarget.name : null;
        var viewer = this.viewer;
        var provider = viewer.getContentProvider();
        var scrollAmount = MaxNumPageIndicesDisplay - 1;
        
        // we encoded the link's name with either the page's index 
        // number or the scroll indices directive.
        if (name) {
            if(name == "frontPage") {
                var newPos = Math.max(0, (this.vcStartPageDisplay - scrollAmount));
                this._updateVCPageInfoDisplay(viewer, provider, newPos);
            }
            else if(name == "backPage") {
                var newPos = this.vcStartPageDisplay + scrollAmount;
                this._updateVCPageInfoDisplay(viewer, provider, newPos);
            }
            else {
                var index = parseInt(name);
                this.viewer.performIndexPage(index);
            }
        }
    },
    
    // Summary:
    //    In a virtual content setting, displays the page indices. Each page index
    //    is selectable and allows the user to directly go to that page. The number of 
    //    indices to display is governed by the constant variable, MaxNumPageIndicesDisplay.
    //
    _updateVCPageInfoDisplay: function(viewer, provider, newStartPagePos)
    {
        var pageThreshold = Math.floor((MaxNumPageIndicesDisplay / 2));
        var nPages = provider.vcGetTotalPageCount();
        var curPageIndex = provider.vcGetCurrentPageIndex();
        var startPage = 0;
        
        if (nPages > MaxNumPageIndicesDisplay) {
            // If there's an advisory start page index specified, used that. Otherwise
            // we'll calculate that based on the current page index.
            startPage = (typeof newStartPagePos != "undefined") ? newStartPagePos : ((curPageIndex - pageThreshold >= 0) ? curPageIndex - pageThreshold : 0);
            if (startPage > 0 && startPage + MaxNumPageIndicesDisplay > nPages) {
                startPage = nPages - MaxNumPageIndicesDisplay;
            }
        }
        // store the current start page index
        this.vcStartPageDisplay = startPage;
                
        // generate HTML string for the page indices display
        var endPage = ((nPages > MaxNumPageIndicesDisplay) ? startPage + MaxNumPageIndicesDisplay : nPages);
        var frontPaging = (startPage > 0);
        var backPaging = (nPages > endPage);
        var htmlStr = "|&nbsp;";
        
        // make page index 1 based.
        startPage++;
        curPageIndex++;
        endPage++;
        
        // There are more page index to the left. Show
        // '...' to let the user to scroll
        if(frontPaging) {
            htmlStr += "<a name=\'frontPage\'>...</a>"
        }
        for(var i = startPage; i < endPage; i++) {
            var isCurPage = (i == curPageIndex);
            var indexStr = isCurPage ? ("<b>" + i + "</b>") : i;
            var btag = isCurPage ? "<span " : "<a ";  // current page need not to be clickable.
            var etag = isCurPage ? "</span>" : "</a>";
            
            // add a small margin if the index is single digit.
            var className = (i < 10) ? "class=\'page-index-text\' " : "";
            htmlStr += (btag + className + "name=\'" + (i - 1) + "\'>" + indexStr + etag);
              
            if(i + 1 < endPage) {
                htmlStr += ",";
            }
        }
        if(backPaging) {
            htmlStr += "<a name=\'backPage\'>...</a>";
        }
        htmlStr += "&nbsp;|"

        // load the html into the page info area and configure the
        // links for the page indexing.
        for (var i = 0; i < this.pageCtrlInfo.length; i++) {
            this.pageCtrlInfo[i].innerHTML = htmlStr;
            this.pageCtrlInfo[i].title = this.labelGotoPage;
            
            var links = dojo.query("a", this.pageCtrlInfo[i]);
            var length = links.length;
            for(var j = 0; j < length; j++) {
                var link = links[j];
                link.setAttribute("href", "javascript:{}");
                this.connect(link, "onclick", "_handlePageIndexClicked");
            }
        }
    },
    
    _updatePageInfoDisplay: function(viewer, provider)
    {
        var total = viewer.getContentItemCount();
        var rangeStart = viewer.getCurrentPageStartItem();
        var span = (rangeStart + viewer.getContentPageSize() + viewer.getHiddenRowsSize());
        var rangeEnd = (viewer.getContentPageSize() > -1) ? Math.min(total, span) : total;
        var offsetPos = (total > 0) ? 1 : 0;
        var msg;
        
        // if the viewer is doing filtering, we have to take into the account
        // of the items that were filtered out. The item shown may not be the total
        // page size, so we have to use the current counts for filtered and shown items.
        //
        if (viewer.getFilter() != null) {
            var displayTotal = viewer.getCurrentDisplayItemsCount();
            var start = (viewer.nItemsShown > 0) ? (rangeStart + offsetPos) : 0;
            var end = (start > 0) ? start + viewer.nItemsShown - 1 : 0;
            msg = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_pageRangeFiltered", start, end, displayTotal, (total - displayTotal));
        }
        else {
            msg = MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_pageRange", (rangeStart + offsetPos), rangeEnd, total);
        }
        for (var i = 0; i < this.pageCtrlInfo.length; i++) {
            this.pageCtrlInfo[i].innerHTML = msg;
        }
    },
    
    // Summary:
    //    Updates the paging control display. This is to show the current page information.
    //
    updatePageControlDisplay: function()
    {
    	var viewer = this.viewer;
        var provider = viewer.getContentProvider();
        
        // don't need to update the paging display.
        if(!viewer.getIsShowingPagingControl())   
            return;

        // show the indices of the content's page if the content
        // is virtual.
        if (provider.isVirtualContent()) {
            this._updateVCPageInfoDisplay(viewer, provider);
        }
        else {
            this._updatePageInfoDisplay(viewer, provider);
        }
        
        var hasPrevPage = viewer.getHasPreviousPage();
        var hasNextPage = viewer.getHasNextPage();
        
        var length = this.pageCtrlPrev.length;
        for(var i = 0; i < length; i++) {
        	var pageCtrlP = this.pageCtrlPrev[i];
        	var pageCtrlN = this.pageCtrlNext[i];
        	var pageCtrlF = this.pageCtrlFirst[i]; 
            var pageCtrlL = this.pageCtrlLast[i]; 
        	
            pageCtrlP.disabled = !hasPrevPage;
            pageCtrlN.disabled = !hasNextPage;
            pageCtrlL.disabled = !hasNextPage; 
            pageCtrlF.disabled = !hasPrevPage; 
            
            pageCtrlP.className = (pageCtrlP.disabled) ? "action-link-disabled" : "action-link";
            pageCtrlN.className = (pageCtrlN.disabled) ? "action-link-disabled" : "action-link";
            
            pageCtrlL.className = (pageCtrlL.disabled) ? "action-link-disabled-abs" :"action-link-last"; 
            pageCtrlF.className = (pageCtrlF.disabled) ? "action-link-disabled-abs" : "action-link";                           
        }
    },
    
    clearSelectedPageSize: function()
    {
        if(this.selectedPageSize) {
            delete this.selectedPageSize;
        }
    },
    
    _createPageRangeControl: function(node)
    {
        var provider = this.viewer.getContentProvider();
        var content = this.viewer.getContent();
        var rows = provider.getContentObjects(content);
        var count = rows ? rows.length : 0;
        var contentCount = count;
        var itemsPerPage = [10, 25, 50, 75, 100];
        
        var doc = document;
		var pageLabel = doc.createElement("label");		
		var pageLabelInfo = doc.createElement("span");
		var rageSelectId = this.viewer.domNode.id + "-select-page-range";
        
        // the content is virtual, get the count from the provider.
        if(provider.isVirtualContent()) {
           count = provider.vcGetContentItemCount();
        }
        pageLabelInfo.style.display = "none";
		pageLabelInfo[_textContent]=this.labelPageRange;
		pageLabel.appendChild(pageLabelInfo);
		pageLabel.setAttribute("for", rageSelectId);
		
		this.pageRangeSelect = doc.createElement("select");
        this.connect(this.pageRangeSelect, "onchange", "_handlePageRangeChanged");
		this.pageRangeSelect.id = rageSelectId;
        
        var pos = 0;
        var initSize = 0;
        
        var length = itemsPerPage.length;
        for(var i = 0; i < length; i++) {
        	var itemsPP = itemsPerPage[i];
            if(count > itemsPP) {
                var option = new Option(itemsPP + "");
                option.value = itemsPP;
                this.pageRangeSelect.options[pos++] = option;
                
                // there's an user set size, retain it.
                if(this.selectedPageSize && this.selectedPageSize == itemsPP) {
                    option.selected = true;
                    initSize = itemsPP;
                }      
            }
            else {
                break;
            }
        }
        
        // WI 32649: User does not want to lose the Select All setting when 
    	// filtering, sorting or refreshing the viewer. In order to prevent
    	// performance issues, instead of Show All it will return the MAX page
    	// size. However, if the result count is less than the MAX page size,
        // we can preserve the Show All setting.
        if(!this.selectedPageSize && this.viewer.sizeSelectedByUser) {
        	var previousPageSizeSelected = this.viewer.getContentPageSize();
            if(previousPageSizeSelected == TableConstants.MAX_PAGE_SIZE && count < previousPageSizeSelected)
                this.selectedPageSize = -1;
            else this.selectedPageSize = TableConstants.MAX_PAGE_SIZE;
        }
        
        // a large number of items can lead to performance problems in the web browser.
        // only provide the "Show All" option when the total number of rows is less
        // than the maximum number of items per page as specified in the itemsPerPage
        // array defined above.
        if (this.pageRangeSelect.options.length < itemsPerPage.length) {
            var showAllOpt = new Option(this.tableViewer_showAllItems);
            showAllOpt.value = -1;
            this.pageRangeSelect.options[pos++] = showAllOpt;

            // there's an user set size, retain it.
            if(this.selectedPageSize && this.selectedPageSize == -1) {
                showAllOpt.selected = true;
                initSize = -1;
            }
        }
        
        // there's no determined user selected page size yet, find the
        // best fit page size for the current content size.
        if (initSize == 0) {
            var i = 0;
            var options = this.pageRangeSelect.options;
            var optLen = options.length - 1;
            var contentCount = rows ? rows.length : 0;
            
            if (this.selectedPageSize) {
                var selectedSize = (this.selectedPageSize == -1) ? contentCount : this.selectedPageSize;
                for (; i < optLen; i++) {
                    var val = parseInt(options[i].value);
                    val = (val == -1) ? contentCount : val;
                    if (val >= selectedSize) {
                        break;
                    }
                }
                // if we can't find best max value, just revert back to min.
                i = (i > optLen) ? 0 : i;
            }
            options[i].selected = true;
            initSize = parseInt(options[i].value);
        }
        if(this.pageRangeSelect.options.length == 1) {
            this.pageRangeSelect.disabled = true;
        }  		
		
		var lang = dojo.locale.split('-')[0];
		var RTL = {ar:true,fa:true,he:true,ur:true,yi:true,iw:true};
        var fragment = doc.createDocumentFragment();
		if (RTL[lang]) {
			fragment.appendChild(doc.createTextNode(this.tableViewer_itemsPerPage));
			fragment.appendChild(pageLabel);
			fragment.appendChild(this.pageRangeSelect);
		}
		else {
			fragment.appendChild(pageLabel);
			fragment.appendChild(this.pageRangeSelect);
			fragment.appendChild(doc.createTextNode(this.tableViewer_itemsPerPage));
		}
		node.appendChild(fragment);
        
        // initialize the viewer's page size to the first item.
        this.viewer.setContentPageSize(initSize);    
    },
    
    _handlePageRangeChanged: function()
    {
        var howMany = -1;
        
        var length=this.pageRangeSelect.options.length;
        var options=this.pageRangeSelect.options;
        for(var i = 0; i < length; i++) {
            if(options[i].selected) {
                howMany = parseInt(options[i].value);
                break;
            }
        }
        this.viewer.setContentPageSize(howMany);   
        this.viewer.curPageItemStart = 0;

        // record the user set page size. Retain this setting for the 
        // table's content changed and the controls are being recreated.
        this.selectedPageSize = howMany;

        var provider = this.viewer.getContentProvider();
        this.viewer.sizeSelectedByUser = true;
        if(provider.isVirtualContent()) {
            provider.vcSetPageSize(howMany);
        }
        else {
            this.viewer._loadTableContentsWithProgress();
        }
        this.updatePageControlDisplay();                   
    },
    
    _createPageControls: function(pageCtrl)
    {
    	var doc=document;
    	var pageCtrlFirst = doc.createElement("button");  
        var pageCtrlPrev = doc.createElement("button");
        var pageCtrlInfo = doc.createElement("span");
        var pageCtrlNext = doc.createElement("button");        
        var pageCtrlLast = doc.createElement("button"); 
        
        var imageFirst  = doc.createElement("img");
        var imageLast  = doc.createElement("img");
                
        pageCtrlPrev.innerHTML = this.labelPrevious;
        pageCtrlNext.innerHTML = this.labelNext;    
                     
        pageCtrlFirst.appendChild(imageFirst);
        pageCtrlLast.appendChild(imageLast);
        pageCtrlFirst.title = this.labelFirstPage;
        pageCtrlLast.title = this.labelLastPage;
        
        imageFirst.setAttribute("src",ImageCache.FIRST_PAGE_ICON); 
        imageLast.setAttribute("src",ImageCache.LAST_PAGE_ICON);
        
        pageCtrlPrev.className = "action-link";
        pageCtrlNext.className = "action-link";
        pageCtrlFirst.className = "action-link"; 
        pageCtrlLast.className  = "action-link-last"; 
        
        
        this.connect(pageCtrlPrev, "onclick", "_handlePagePrev");
        this.connect(pageCtrlNext, "onclick", "_handlePageNext");
        this.connect(pageCtrlFirst,"onclick", "_handlePageFirst"); 
        this.connect(pageCtrlLast,"onclick", "_handlePageLast"); 
        
        
        var fragment = doc.createDocumentFragment();
        fragment.appendChild(pageCtrlFirst);          
        fragment.appendChild(pageCtrlPrev);
        fragment.appendChild(pageCtrlInfo);
        fragment.appendChild(pageCtrlNext);
        fragment.appendChild(pageCtrlLast);         
		pageCtrl.appendChild(fragment);
		
		
		this.pageCtrlFirst.push(pageCtrlFirst); 
        this.pageCtrlLast.push(pageCtrlLast); 
       
        this.pageCtrlPrev.push(pageCtrlPrev);
        this.pageCtrlInfo.push(pageCtrlInfo);
        this.pageCtrlNext.push(pageCtrlNext);
    },
    
    
    _handlePageFirst: function()
    {
        if (this.viewer.getHasPreviousPage())
        {
            this.viewer.performFirstPage();
            this.updatePageControlDisplay();
        }       
    },
 
    
    _handlePageLast: function()
    {
      if(this.viewer.getHasNextPage()) 
      {
          this.viewer.performLastPage();
          this.updatePageControlDisplay();            
       }
    },    
    
    
    _handlePagePrev: function()
    {
        if(this.viewer.getHasPreviousPage()) {
            this.viewer.performPreviousPage();
            this.updatePageControlDisplay();
        }
    },
    
    _handlePageNext: function()
    {
        if(this.viewer.getHasNextPage()) {
            this.viewer.performNextPage();
            this.updatePageControlDisplay();            
        }
    },
    
    setGroupControlEnabled: function(/*boolean*/ enabled)
    {
    	this.groupSelect.disabled = !enabled;
    },
    
    setFilterControlEnabled: function(/*boolean*/ enabled)
    {
    	this.filterInput.disabled = !enabled;
    	this.resetFilter.disabled = !enabled;	
    },
    
    resetGroupBy: function()
    {
        if (this.groupSelect) {
            var options = this.groupSelect.options;
            var length = options.length;
            if (length > 0) {
                options[0].selected = true;
                for (var i = 1; i < length; i++) {
                    options[i].selected = false;
                }
            }
        }      
    },
    
    _constructGroupControl: function(controlNode)
    {
        var viewer = this.viewer;
        var provider = viewer.getContentProvider();
        var content = viewer.getContent();
        var nColumns = provider.getNumColumns(content);
        var hasGroupables = false;
        
        for (var i = 0; i < nColumns; i++) {
            // only allow visiable columns to be groupable.
            if (provider.getIsColumnGroupable(content, i) && viewer.isColumnVisible(i)) {
                hasGroupables = true;
                break;
            }    
        }
        if(!hasGroupables) {
            return;
        }
        var doc=document;
		var selectLabel = doc.createElement("label");
		
        this.groupSelect = doc.createElement("select");
        this.groupSelect.id = "table-groupby-select-" + viewer.domNode.id;
        this.connect(this.groupSelect, "onchange", "_handleGroupBy");
		
        // create an empty selection
		var pos = 0;
		if (!viewer.hideUngrouped) {
            this.groupSelect.options[0] = new Option(this.tableViewer_ungrouped);
            this.groupSelect.options[0].selected = true;
            this.groupSelect.options[0].value = -1;
            pos++;
        }
        for (var i = 0; i < nColumns; i++) {
            if (provider.getIsColumnGroupable(this.content, i) && viewer.isColumnVisible(i)) {
                var heading = provider.getColumnHeading(content, i);
                var option = new Option(heading);
                option.value = i;
				if(this.viewer.groupByColumn && this.viewer.groupByColumn == i)
					option.selected = true
				
                this.groupSelect.options[pos++] = option;
            }    
        }
		selectLabel.appendChild(doc.createTextNode(this.tableViewer_groupBy + " "));
        selectLabel.appendChild(this.groupSelect)
        selectLabel.setAttribute("for", this.groupSelect.id);
        controlNode.appendChild(selectLabel);
		dojo.addClass(controlNode, "content-control-area");
    },

    _handleGroupBy: function()
    {
        var groupByCol = -1;
        var options=this.groupSelect.options;
        var length=options.length;
        for(var i = 0; i < length; i++) {
            if(options[i].selected) {
                groupByCol = parseInt(options[i].value);
                break;
            }
        }
        this.viewer.performGroupBy(groupByCol);
    },
    
    _handleViewModeChange: function()
    {
    	var viewModeId = -1;
    	var options=this.viewModeSelect.options;
    	var length=options.length;
        for(var i = 0; i < options.length; i++) {
            if(options[i].selected) {
                viewModeId = options[i].value;
                break;
            }
        }
        this.viewer.switchViewMode(viewModeId);
    },
    
    _constructFilterControls: function(node)
    {
        var container = node;
        var doc = document;
		var inputLabel = doc.createElement("label");
		var inputLabelInfo = doc.createElement("span");
        var filterId = this.viewer.domNode.id + "-filter-input";
        
		inputLabelInfo.style.display = "none";
        
		if (this.viewer.tableSummary) {
			inputLabelInfo[_textContent]=this.viewer.tableSummary + " " + this.labelFilterInput;
		}
		inputLabel.appendChild(inputLabelInfo);
		inputLabel.setAttribute("for", filterId);
		
		this.filterInput = doc.createElement("input");
        this.filterInput.type = "text";
        this.filterInput.size = 22;
        this.filterInput.id = filterId;

        this.connect(this.filterInput, "onkeyup", "_filterInputSet");
        this.connect(this.filterInput, "onfocus", "_filterInputFocus");
        this.connect(this.filterInput, "onblur", "_filterInputBlur");
		        
        this.resetFilter = doc.createElement("button");
        var img = doc.createElement("img");
        
        img.src = ImageCache.CLEAR_FILTER_ENABLED;
        this.resetFilter.appendChild(img);
        
        dojo.addClass(this.resetFilter, "action-link");
        dojo.addClass(this.resetFilter, "clear-filter");
		if (this.viewer.tableSummary) {
			this.resetFilter.name = this.viewer.tableSummary + " " + Constants.getMessage("tableViewer_filterClean");
		}
		this.resetFilter.title =  Constants.getMessage("tableViewer_filterClean");
        this.connect(this.resetFilter, "onclick", "_resetFilter");
        
        var fragment = doc.createDocumentFragment();
        
        fragment.appendChild(inputLabel);
		fragment.appendChild(this.filterInput);
        fragment.appendChild(this.resetFilter);
		container.appendChild(fragment);

        this._setFilterInputInactive();
    },

    _constructViewModeControl: function(controlNode)
    {
        var viewModeList = this.viewer.viewModeList;
        if (viewModeList == null || viewModeList.length < 2 ) {
        	return;
        }
        
        var doc=document;
		var selectLabel = doc.createElement("label");
        this.viewModeSelect = doc.createElement("select");
        
        var currentViewModeId = this.viewer.getCurrentViewModeId();
        
        var length=viewModeList.length;
        for (var i = 0; i < length; i++) {
        	var viewMode = viewModeList[i];
        	var viewOption;
        	// JS api is :   new Option([text], [value], [defaultSelected], [selected])
        	if (viewMode.id == currentViewModeId) {
        		viewOption = new Option(viewMode.label, viewMode.id, true, true);
        	} else {
        		viewOption = new Option(viewMode.label, viewMode.id);
        	}
        	this.viewModeSelect.options[i] = viewOption;
        }
        
        this.connect(this.viewModeSelect, "onchange", "_handleViewModeChange");
        
		selectLabel.appendChild(this.viewModeSelect);
		controlNode.appendChild(doc.createTextNode(" " + this.tableViewer_viewModeSelectLabel + " "));
        controlNode.appendChild(selectLabel);
		dojo.addClass(controlNode, "content-control-area");
    },
    
    _resetFilter: function()
    {
        this.viewer.performFilterBy(null);
        this._setFilterInputInactive();
    },

    _filterInputSet: function(evt)
    {
        evt = evt || window.event;
        
        // don't interpret TAB key. Tab is for keyboard
        // traversal. let it 'pass' by the filter input.
        if(evt && (evt.keyCode == dojo.keys.TAB)) {
            return;
        }
        
        // The user has typed more chars, cancel the current
        // start of filtering and wait for more chars.
        //
        if(this.delayFilterTimer) {
            clearTimeout(this.delayFilterTimer);
            this.delayFilterTimer = null;
        }
        
        // set a small delay before doing the filtering.
        // If the user typed more chars, we want to queue up
        // more filter chars. Otherwise, we would be performing 
        // filtering on every char.
        this.delayFilterTimer = setTimeout(dojo.hitch(this, "_doFilter"), FilterInputDelay);        
    },
   
    _doFilter: function()
    {   
        this.delayFilterTimer = null;
        this.filterText = this.filterInput.value;
            
        // make it a null to indicate clearing of the filter.
        if(this.filterText && this.filterText.length == 0)
            this.filterText = null;
        
        this.viewer.performFilterBy(this.filterText);
		this.filterInput.title = "";
    },
    
    _filterInputFocus: function()
    {
        this.filterInput.className = "";
        var hasFilter = this.viewer.getFilter() != null;
        this.filterInput.className = "filter-text-input";
        
        if(this.isCJK) {
            dojo.addClass(this.filterInput, "filter-text-input-cjk")
        }
        
        // we don't have a filter in the viewer. Clear the prompt
        // we have displayed in the input field.
        if(!hasFilter)
            this.filterInput.value = "";        
        
    },
    
    _filterInputBlur: function()
    {
        this._setFilterInputInactive();
    },
    
    _setFilterInputInactive: function()
    {
        this.filterInput.className = "filter-text-input-disabled";
        
        if(this.isCJK) {
            dojo.addClass(this.filterInput, "filter-text-input-cjk")
        }
        var noFilter = this.viewer.getFilter() == null;
        
        // we don't have a filter in the viewer, show the
        // default prompt in the filter field.
        if (noFilter) {
            var txt = this.labelFilterPrompt;
			this.filterInput.value = txt;
			this.filterInput.title = txt;
		}   
    },

    // Summary:
    //    Internal method to connect a node's event to an internal function. 
    connect: function(node, event, func)
    {
        if(!this.connections) {
            this.connections = [];
        }
        var cnt = dojo.connect(node, event, this, func);  
        this.connections.push(cnt);  
    },
    
    disconnectAll: function()
    {
        if(!this.connections) return;
        
        var len = this.connections.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            d.disconnect(this.connections[i]);
        }
        delete this.connections;    
    }
});

})();
