/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
package com.ibm.rational.test.lm.spi.bf.internal;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import com.buildforge.services.client.dbo.Server;
import com.ibm.ccl.soa.deploy.operation.RunState;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIAuthenticationException;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIClientConnectionException;
import com.ibm.rational.test.lm.spi.bf.common.BFAPIException;
import com.ibm.rational.test.lm.spi.bf.common.BFConstants;
import com.ibm.rational.test.lm.spi.bf.common.DiscoveryUnitsMapKey;
import com.ibm.rational.test.lm.spi.common.SPIException;

public class DiscoveryThread extends Thread {

	private ConcurrentHashMap<DiscoveryUnitsMapKey, DiscoveryUnit> discoveryUnitsMap = new ConcurrentHashMap<DiscoveryUnitsMapKey, DiscoveryUnit>();

	private static DiscoveryThread instance = null;
	
	/*
	 * Before scanning discoveryUnitsMap, thread needs to check this variable.
	 * If it is false, that means there is no new discovery unit added to the
	 * map after the scan last time and all of units in the map have been
	 * completed, so the thread should sleep to wait for new one coming. When
	 * new discovery unit adding to the map, the variable would be set to true
	 * if it was false.
	 */
	private volatile boolean addNew = false;
	
	private DiscoveryThread() {
		super();
	}

	public static synchronized DiscoveryThread getInstance() {
		if (instance == null) {
			instance = new DiscoveryThread();
		}
		return instance;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	@Override
	public void run() {
		while (true) {
			if (!addNew) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
			} else {
				if (!discoveryUnitsMap.isEmpty()) {
					addNew = false;
					Set<DiscoveryUnitsMapKey> keys = null;
					synchronized (discoveryUnitsMap) {
						keys = discoveryUnitsMap.keySet();
					}
					for (DiscoveryUnitsMapKey key : keys) {
						DiscoveryUnit unit = null;
						synchronized (discoveryUnitsMap) {
							unit = discoveryUnitsMap.get(key);
						}
						if (unit != null) {
							if (unit.isCanceled() || unit.isCompleted()) {
								continue;
							}
							executeDiscovery(key.getHostname(), key.getPort(), unit);
						}
					}					
				} else {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
					}
				}
			}
		}
	}

	public void addDiscoveryUnit(DiscoveryUnitsMapKey key, DiscoveryUnit unit) {
		synchronized (discoveryUnitsMap) {
			discoveryUnitsMap.put(key, unit);
		}
		if (!addNew) {
			addNew = true;
		}
	}

	public void cancelDiscoveryUnit(DiscoveryUnitsMapKey key) {
		DiscoveryUnit unit = null;
		synchronized (discoveryUnitsMap) {
			unit = discoveryUnitsMap.get(key);
		}
		if (unit != null) {
			unit.setCanceled(true);
		}
	}

	public Integer getDiscoveryUnitStatus(DiscoveryUnitsMapKey key) {
		DiscoveryUnit unit = null;
		synchronized (discoveryUnitsMap) {
			unit = discoveryUnitsMap.get(key);
		}
		if (unit != null) {
			if (unit.isCanceled()) {
				return RunState.SUSPENDED;
			}
			if (unit.isCompleted()) {
				synchronized (discoveryUnitsMap) {
					discoveryUnitsMap.remove(key);
				}
				return RunState.STOPPED;
			} else {
				return RunState.RUNNING;
			}
		} else {
			return RunState.UNKNOWN;
		}
	}
	
	/*
	 * Only be invoked when the unit is canceled
	 */
	public String[] getDiscoveredServerExIdsOnCanceled(DiscoveryUnitsMapKey key) {
		List<String> externalIds = new ArrayList<String>();
		DiscoveryUnit unit = null;
		synchronized (discoveryUnitsMap) {
			unit = discoveryUnitsMap.get(key);
		}
		if (unit != null && unit.isCanceled()) {
			int discoveredNum = unit.getDiscoveredNum();
			List<Server> servers = unit.getServers();
			for (int index = 0; index < discoveredNum; index++) {
				String serverId = String.valueOf(servers.get(index)
						.getServerId());
				externalIds.add(serverId);
			}
			// Remove it
			synchronized (discoveryUnitsMap) {
				discoveryUnitsMap.remove(key);
			}
		}
		return externalIds.toArray(new String[externalIds.size()]);
	}
	
	/**
	 * Get the APIClientConnection as SystemConnection
	 * 
	 * @return APIClientConnection systemConnection
	 * @throws BFAPIClientConnectionException
	 * @throws BFAPIAuthenticationException
	 */
	private BFAPIClientConnection getBFAPIConnection(
			String provider_server_name, int provider_server_port)
			throws BFAPIException{
		BFAPIClientConnection BFAPIConn = ConnectionController
				.getSystemConnection(provider_server_name, provider_server_port);
		BFAPIConn.connect();
		return BFAPIConn;
	}

	
	private void executeDiscovery(String hostname, int port, DiscoveryUnit unit) {
		List<Server> servers = unit.getServers();
		int discoveredNum = unit.getDiscoveredNum(); // default value is 0
		for (Server server : servers) {
			if (unit.isCanceled()) {
				break;
			}
			try {
				discoverManifest(hostname, port, server);
				unit.setDiscoveredNum(++discoveredNum);
			} catch (Exception e) {
				// TODO add log
			}
		}
		unit.setCompleted(true);
	}
	
	private void discoverManifest(String hostname, int port, Server server) 
			throws SPIException {
		BFAPIClientConnection connection = null;
		try {
			connection = getBFAPIConnection(hostname,
					port);
			BuildForgeArtifacts bfArtifacts = new BuildForgeArtifacts();
			// Update collector for discovery
			if (server.getCollectorId() != null
					&& server.getCollectorId().length() > 0) {
				bfArtifacts.updateCollectorForDiscovery(connection
						.getConnection(), server.getCollectorId());
			} else {
				server.setCollectorId(BFConstants.agentCollectorId);
				server.update();
			}
			if (bfArtifacts.isServerAlive(server, connection.getConnection())) {
				server.refreshManifest(Server.DEFAULT_TEST_TIMEOUT);
			}
		} catch (Exception e) {
			APIExceptionHandle.handleAPIExceptions(e);
		} finally {
			try {
				// Release the Build Forge connection resource to avoid the
				// connection leak
				if (connection != null) {
					connection.close();
				}
			} catch (BFAPIClientConnectionException e) {
				// Do not need to throw exception for it is not a big issue.
			}
		}
	}
}
